const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var gCapture = null;
function captureLoad() 
{
  gCapture = new Capture();
  gCapture.init();
};

function captureUnload() 
{  
  gCapture.destroy();
  gCapture = null;
};

function Capture() {};
Capture.prototype = {
  list: null,
  _default: null,
  _branch: null,
  _text: null,
  _overlays: null,
  _loads: null,
  _accepts: null,
  _index: 0,
  _referrer: null,
  _changed: false,
  
  init: function()
  {
    //close window if nothing passed
    if (window.arguments.length == 0)
      window.close();
    
    //close window if no builders available
    var catMgr = Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager);
    var entries = catMgr.enumerateCategory("urlnav-builders");
    if (!entries.hasMoreElements())
      window.close();
        
    //get preference branch
    var pbs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService); 
    this._branch = pbs.getBranch("urlnav.");
    this._default = this._branch.getCharPref("builder.default");
    this._referrer = document.getElementById("checkReferrer");
    this._referrer.setAttribute("checked", (this._branch.getBoolPref("referrer.send")) ? "true" : "false");
    
    //set url
    this._text = document.getElementById("urlText");
    this._text.setAttribute("value", window.arguments[0]);
      
    //loop through builders
    this.list = document.getElementById("builderList");
    do {
      //create builder xpcom
      var entry = entries.getNext().QueryInterface(Components.interfaces.nsISupportsCString).data;
      var contract = catMgr.getCategoryEntry("urlnav-builders", entry);
      var builder = Components.classes[contract].createInstance(Components.interfaces.navIBuilder);    

      //create builder element
      var item = document.createElementNS(XUL_NS, "builder");
      this.list.appendChild(item);
      item.component = builder;
      item.setAttribute("id", contract);
            
      //save overlay
      if (builder.overlay != "")
        this.registerOverlay(builder.overlay);
         
      //select the default
      if (contract == this._default)
        this.list.selectedItem = item;
        
    } while (entries.hasMoreElements());

    //load overlays
	  this.observe("","","");
  },
  
  guess: function()
  {
    if (this.list.selectedIndex < 0)
    	return;
    	
    if (!this._changed) {
   	  this.list.selectedItem.component.guess(this._text);
   	  this.urlChanged(false);
   	};
  },
  
  destroy: function()
  {
    this.list = null;  
    this._default = null;
    this._branch = null;
    this._text = null;
    this._overlays = null;
    this._loads = null;
    this._accepts = null;
    this._index = null;
    this._referrer = null;
    this._changed = null;
  },
    
  registerOverlay: function(aURL)
  {
    if (!this._overlays)
      this._overlays = new Array();
      
    this._overlays.push(aURL);
  },
  
  registerCallback: function(aType, aFunction)
  {
    switch (aType) {
      case "load":
        if (!this._loads)
          this._loads = new Array();
        this._loads.push(aFunction);
        break;
      case "accept":
        if (!this._accepts)
          this._accepts = new Array();
        this._accepts.push(aFunction);
        break;        
    };
  },
  
  observe: function(aSubject, aTopic, aData)
  {
    //load overlay
    if (this._index < this._overlays.length) {
      var overlay = this._overlays[this._index];
      this._index++;
      document.loadOverlay(overlay, this);
      
    //fire load callbacks
    } else {
    	if (this._loads) {
        for (var i=0; i<this._loads.length; i++)
          this._loads[i]();        
	    };
	    
	    //update url changes
	    this.list.selectedItem.component.guess(this._text);
	    this.urlChanged(false);
	    
	    //do auto
	    if (window.arguments[3])
	    	document.documentElement.acceptDialog();
    };
  },
  
  urlChanged: function(aChanged)
  {
    if (aChanged)
      this._changed = true;
      
    var start = document.getElementById("startText");
    var end = document.getElementById("endText");   
    var value = document.getElementById("valueText");
    
    start.setAttribute("value", this._text.selectionStart);
    end.setAttribute("value", this._text.selectionEnd);
    value.setAttribute("value", this._text.value.substring(start.value, end.value));
  },
  
  accept: function()
  {
    var bundle = document.getElementById("captureBundle");
    var ps = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
    
    //validate selection
    if (this._text.selectionStart == this._text.selectionEnd) {
      ps.alert(window, bundle.getString("urlnav.error.title"), bundle.getString("urlnav.error.selection"));
      return false
    };
    
    //fire accept callbacks
    if (this._accepts) {
      for (var i=0; i<this._accepts.length; i++) {
        var value = this._accepts[i]();
        if (!value)
          return false;
      };
    };
    
    //do a test increment
    var builder = this.list.selectedItem.component;
    var start = document.getElementById("startText").value;
    var end = document.getElementById("endText").value;    
    try {
      builder.increment(document.getElementById("valueText").value, Components.interfaces.navIList.MOVE_UP);
    } catch(e) {
      ps.alert(window, bundle.getString("urlnav.error.title"), e.message);
      return false;
    };
        
    //update list
    var url = window.arguments[0];
    var referrer = (this._referrer.getAttribute("checked") == "true") ? window.arguments[2] : null;
    window.arguments[1].init(url, builder, start, end, referrer);
    return true;      
  }
};